##############################################################################
# Generate an N by 4 MA(1) series with coefficient matrix a1
# X(t) = e(t) + a1 * e(t-1), t = 1..N, so need (N+1) rnorms
##############################################################################

ma1.sim <- function( N, a1=diag(4) )
{
	e <- matrix(rnorm(4*(N+1)), nrow=4)
	x <- e[,2:(N+1)] + a1 %*% e[,1:N]
	t(x)
}

##############################################################################
# Generate an N by 4 MA(2) series with coefficient matrices a1 and a2
# X(t) = e(t) + a1 * e(t-1) + a2 * e(t-2), t = 1..N, so need (N+2) rnorms
##############################################################################

ma2.sim <- function( N, a1=diag(4), a2=diag(4) )
{
	e <- matrix(rnorm(4*(N+2)), nrow=4)
	x <- e[,3:(N+2)] + a1 %*% e[,2:(N+1)] + a2 %*% e[,1:N]
	t(x)
}

##############################################################################
# ma.power computes Prob{ accept H_0 | H_1 or H_0 }, where 
# H_0 : f_11 = f_22, and H_1 : f_11 != f_22.  The test statistics are
# the trace and the determinant of the matrix (W1 * (W2)^(-1)).  The
# independent test uses the histograms generated by Monte Carlo
# simulations.  They are wishart.trace and wishart.det.  The dependent
# test uses the histogram generated according to the estimate of the
# spectral density matrix.
#
# N : number of repetitions
# K : number of points in a group for averaging the d(x)
# np : length of the sample.  The length of the histogram will be
#      np%/%(2*K) -1
##############################################################################

ma.power <- function( N=50, K=10, np=2020 )
{
	alpha <- seq(0, 1, by=.2)
	beta <- seq(0, 1, by=.2)

	# For storing the probability
	p1 <- matrix(0, nrow=length(alpha), ncol=length(beta))
	p2 <- matrix(0, nrow=length(alpha), ncol=length(beta))
	p3 <- matrix(0, nrow=length(alpha), ncol=length(beta))
	p4 <- matrix(0, nrow=length(alpha), ncol=length(beta))

	# build matrices here for constructing MA series
	a1 <- matrix(0, 4, 4)
	a1[3:4,1:2] <- diag(2)
	a1[1:2,3:4] <- diag(2)
	
	for (i in 1:length(alpha))
	  for (j in 1:length(beta))
	  {
cat("alpha[", i, "] =", alpha[i], "  beta[", j, "] =", beta[j], "\n")
	    # build the matrix a(1) for MA(1)
	    a1[1:2,1:2] <- c(beta[j], 0,0, beta[j])
	    a1[3:4,3:4] <- c(alpha[i], 0,0, alpha[i])

	    # generate N x's all at once
#	    x <- ma2.sim(np*N, a1, a2)
	    x <- ma1.sim(np*N, a1)
	
	    for (k in 1:N)
	    {
		# extract the k-th sample and compute its spectral matrix
		d <- x[((k-1)*np+1):(np*k),]	
		f <- est.spectral.matrix(d[,1], d[,2], d[,3], d[,4],K=K)

		# compute the trace and determinant statistics from f
		tr.f <- trace.stat(f)
		det.f <- det.stat(f)

		trace1 <- kstwo(tr.f, wishart.trace[[K]])
		det1 <- kstwo(det.f, wishart.det[[K]])
	
		# generate a sample with spectral matrix f
		y <- sim.dep(f, np)
		g <- est.spectral.matrix(y[,1], y[,2], y[,3], y[,4],K=K)

		trace2 <- kstwo(tr.f, trace.stat(g))
		det2 <- kstwo(det.f, det.stat(g))
	
		if (trace1 > 0.05) p1[i,j] <- p1[i,j] + 1
		if (trace2 > 0.05) p2[i,j] <- p2[i,j] + 1
		if (det1 > 0.05)   p3[i,j] <- p3[i,j] + 1
		if (det2 > 0.05)   p4[i,j] <- p4[i,j] + 1
	    }
	  }
	list(tr.indep=p1/N, tr.dep=p2/N, det.indep=p3/N, det.dep=p4/N)
}

##############################################################################




